@@ -8,7 +8,7 @@ from collections import Counter, defaultdict, OrderedDict
 import numpy as np
 import pandas as pd
 
-from pandas import (Index, Series, DataFrame, isnull)
+from pandas import (Index, Series, DataFrame, isna)
 from pandas.compat import lrange
 from pandas import compat
 from pandas.util.testing import assert_series_equal, assert_frame_equal
@@ -306,6 +306,22 @@ class TestSeriesAggregate(TestData):
                           name=self.series.name)
         assert_series_equal(result, expected)
 
+    def test_non_callable_aggregates(self):
+        # test agg using non-callable series attributes
+        s = Series([1, 2, None])
+
+        # Calling agg w/ just a string arg same as calling s.arg
+        result = s.agg('size')
+        expected = s.size
+        assert result == expected
+
+        # test when mixed w/ callable reducers
+        result = s.agg(['size', 'count', 'mean'])
+        expected = Series(OrderedDict([('size', 3.0),
+                                       ('count', 2.0),
+                                       ('mean', 1.5)]))
+        assert_series_equal(result[expected.index], expected)
+
 
 class TestSeriesMap(TestData):
 
@@ -377,8 +393,8 @@ class TestSeriesMap(TestData):
 
         merged = left.map(right)
         assert merged.dtype == np.float_
-        assert isnull(merged['d'])
-        assert not isnull(merged['c'])
+        assert isna(merged['d'])
+        assert not isna(merged['c'])
 
     def test_map_type_inference(self):
         s = Series(lrange(3))
